@@ -5,18 +5,13 @@
  *
  * daniel@veillard.com
  *
- * 15 Nov 2000 ht - modified for VMS
  */
 
 #ifndef __XML_IO_H__
 #define __XML_IO_H__
 
 #include <stdio.h>
-#if defined(WIN32) && defined(_MSC_VER)
-#include <libxml/xmlwin32version.h>
-#else
 #include <libxml/xmlversion.h>
-#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -159,17 +154,9 @@ void	xmlRegisterDefaultInputCallbacks	(void);
 xmlParserInputBufferPtr
 	xmlAllocParserInputBuffer		(xmlCharEncoding enc);
 
-#ifdef VMS
-xmlParserInputBufferPtr
-	xmlParserInputBufferCreateFname		(const char *URI,
-                                                 xmlCharEncoding enc);
-#define xmlParserInputBufferCreateFilename xmlParserInputBufferCreateFname
-#else
 xmlParserInputBufferPtr
 	xmlParserInputBufferCreateFilename	(const char *URI,
                                                  xmlCharEncoding enc);
-#endif
-
 xmlParserInputBufferPtr
 	xmlParserInputBufferCreateFile		(FILE *file,
                                                  xmlCharEncoding enc);
@@ -254,6 +241,7 @@ xmlParserInputPtr xmlNoNetExternalEntityLoader(const char *URL,
 
 xmlChar *xmlNormalizeWindowsPath	(const xmlChar *path);
 
+int	xmlCheckFilename		(const char *path);
 /**
  * Default 'file://' protocol callbacks 
  */
